@@ -1,4 +1,4 @@
-# Copyright (c) 2019 The Regents of the University of Michigan
+# Copyright (c) 2022 The Regents of the University of Michigan
 # All rights reserved.
 # This software is licensed under the BSD 3-Clause License.
 import os
@@ -9,13 +9,17 @@ from signac_dashboard.module import Module
 
 
 class FileList(Module):
-    """Lists files in the job workspace with download links.
+    """Lists files in the job directory with download links.
 
+    :param context: Supports :code:`'JobContext'`.
+    :type context: str
     :param prefix_jobid: Whether filenames should be prefixed with the job id
         when being downloaded (default: :code:`True`).
     :type prefix_jobid: bool
     """
 
+    _supported_contexts = {"JobContext"}
+
     def __init__(
         self,
         name="File List",
@@ -24,7 +28,13 @@ class FileList(Module):
         prefix_jobid=True,
         **kwargs,
     ):
-        super().__init__(name=name, context=context, template=template, **kwargs)
+
+        super().__init__(
+            name=name,
+            context=context,
+            template=template,
+            **kwargs,
+        )
         self.prefix_jobid = prefix_jobid
 
     def download_name(self, job, filename):
