@@ -1,4 +1,4 @@
-# Copyright (c) 2019 The Regents of the University of Michigan
+# Copyright (c) 2022 The Regents of the University of Michigan
 # All rights reserved.
 # This software is licensed under the BSD 3-Clause License.
 from collections import OrderedDict
@@ -9,7 +9,13 @@ from signac_dashboard.module import Module
 
 
 class StatepointList(Module):
-    """Displays the job state point."""
+    """Displays the job state point.
+
+    :param context: Supports :code:`'JobContext'`.
+    :type context: str
+    """
+
+    _supported_contexts = {"JobContext"}
 
     def __init__(
         self,
@@ -18,7 +24,13 @@ class StatepointList(Module):
         template="cards/statepoint_list.html",
         **kwargs,
     ):
-        super().__init__(name=name, context=context, template=template, **kwargs)
+
+        super().__init__(
+            name=name,
+            context=context,
+            template=template,
+            **kwargs,
+        )
 
     def get_cards(self, job):
         sp = OrderedDict(sorted(job.statepoint().items(), key=lambda t: t[0]))
